{*******************************************************}
{                                                       }
{         Copyright (c) FINANCES Ltd                    }
{         2003   ver. 1                                 }
{                                                       }
{         www.finances-ltd.narod.ru                     }
{         finances-ltd@narod.ru                         }
{                                                       }
{         Thanks to Michal Mutl                         }
{                                                       }
{*******************************************************}

unit zPanelButton;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, ExtCtrls;

type
  TzPanelButton = class(TCustomPanel)
  private
    FColor: TColor;
    FBtnWidth: Integer;
    FmouseInControl: Boolean;
    FButtonCaption: string;
    FOnBtnClick: TNotifyEvent;
    FOnChange: TNotifyEvent;
    FText: string;
    FGlyph: TBitmap;
    FOnMouseUp: TMouseEvent;
    FAlignment: TAlignment;
    // Michal Mutl
    FBtnColor,
    FBtnHLColor: TColor;

    procedure CMEnabledChanged (var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure WMSetFocus (var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus (var Message: TWMKillFocus); message WM_KILLFOCUS;
//    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    function GetButtonRect: TRect;
    procedure SetButtonCaption(Value: string);
    procedure SetButtonWidth(Value: Integer);
    procedure SetText(Value: string);
    procedure SetGlyph(Value: TBitmap);
    function GetBitmap: TBitmap;
    // Michal Mutl
    procedure SetButtonColor(const Value: TColor);
    procedure SetButtonHLColor(const Value: TColor);

  protected
    procedure Paint; override;
    procedure PaintButton;
    procedure Loaded; override;
  public
    constructor Create (AOwner: TComponent); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    destructor Destroy; override;
    procedure Click; override;
    procedure Resize; override;
  published
    property ButtonWidth: Integer read FBtnWidth write SetButtonWidth;
// Michal Mutl
    property ButtonColor: TColor read FBtnColor write SetButtonColor;
    property ButtonHighlightColor: TColor read FBtnHLColor write SetButtonHLColor;

    property OnBtnClick: TNotifyEvent read FOnBtnClick write FOnBtnClick;
    property Text: string read FText write SetText;
    property Glyph: TBitmap read GetBitmap write SetGlyph;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property ButtonCaption: string read FButtonCaption write SetButtonCaption;
    property Font;
    property Color;
    property ParentColor;
    property Enabled;
    property Visible;
    property Align;
    property Alignment: TAlignment read FAlignment write FAlignment;
    property Cursor;
    property Hint;
    property ParentShowHint;
    property ShowHint;
    property PopupMenu;
    property TabOrder;
    property TabStop;
    property UseDockManager;
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property DragMode;
    property DragCursor;
    property ParentBiDiMode;
    property DockSite;
    property OnEndDock;
    property OnStartDock;
    property OnCanResize;
    property OnConstrainedResize;
    property OnDockDrop;
    property OnDockOver;
    property OnGetSiteInfo;
    property OnUnDock;
    property OnContextPopup;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp read FOnMouseUp write FOnMouseUp;
//    property OnMouseUp;
    property OnResize;
    property OnStartDrag;
  end;

procedure Register;

implementation


constructor TzPanelButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ParentFont := True;
  ParentColor := False;
  Alignment := taLeftJustify;
  Color := clWindow;
  TabStop := True;
  ControlStyle := ControlStyle + [csAcceptsControls, csOpaque];
  FBtnWidth := 13;
  FBtnColor:=clWhite;
  FBtnHLColor:=$00CEBAB5;
  SetBounds(0, 0, 80, 20);
  FText := '';
  // Michal Mutl
  ControlStyle := ControlStyle - [csSetCaption];// - [csAcceptsControls];

end;

destructor TzPanelButton.Destroy;
begin
  if Assigned(FGlyph) then FGlyph.Free;
  inherited Destroy;
end;

procedure TzPanelButton.CMEnabledChanged (var Message: TMessage);
begin
  inherited;
  if (csDesigning in ComponentState) then Exit;
  if (csLoading in ComponentState) then
    FColor := Color;
  Repaint;
  //if Enabled then
  //  Color := FColor
  //else
  //  Color := clBtnFace;
end;

procedure TzPanelButton.WMSetFocus (var Message: TWMSetFocus);
begin
  FColor := Color;
  Color := clWindow;
  inherited;
end;

procedure TzPanelButton.WMKillFocus (var Message: TWMKillFocus);
begin
  if csDestroying in ComponentState then exit;
  Color := FColor;
  inherited;
end;

procedure TzPanelButton.Click;
begin
  if CanFocus and not Focused then SetFocus;
  inherited;
end;

procedure TzPanelButton.Resize;
begin
  inherited;
end;

procedure TzPanelButton.PaintButton;
  function NewColor(ACanvas: TCanvas; clr: TColor; Value: integer): TColor;
  var r, g, b: integer;
  begin
    if Value > 100 then Value := 100;
    clr := ColorToRGB(clr);
    r := Clr and $000000FF;
    g := (Clr and $0000FF00) shr 8;
    b := (Clr and $00FF0000) shr 16;


    r := r + Round((255 - r) * (value / 100));
    g := g + Round((255 - g) * (value / 100));
    b := b + Round((255 - b) * (value / 100));

    Result := Windows.GetNearestColor(ACanvas.Handle, RGB(r, g, b));
  end;

var
  R: TRect;
  x, y, l, t, w, c: Integer;
  BM: TBitmap;
  PixelColor: TColor;
begin
  R := GetButtonRect;
  InflateRect(R, 1, 0);
  R.Left := R.Left + 2;

  if (csDesigning in ComponentState) or (Enabled and not FmouseInControl) then
  // Michal Mutl
    Canvas.Brush.Color := FBtnColor

  else begin
    if not Enabled then
      Canvas.Brush.Color := clBtnFace
    else Canvas.Brush.Color := FBtnHLColor;//NewColor(Canvas, clHighLight, 68);
  end;
  Canvas.FillRect(R);
  Canvas.Brush.Color := clBlack;
  Canvas.FrameRect(R);

  if Assigned(FGlyph) then begin
    x := (R.Right - R.Left - FGlyph.Width) div 2;
    if x < 0 then x := R.Left
    else x := R.Left + x;
    y := (R.Bottom - R.Top - FGlyph.Height) div 2;
    if y < R.Top + 2 then y := R.Top + 2
    else y := R.Top + y;
    if Enabled then
      Canvas.Draw(x, y, FGlyph)
    else begin
      BM := TBitmap.Create;
      try
        BM.Assign(FGlyph);
        w := BM.Width;
        for l := 0 to w - 1 do
          for t := 0 to w - 1 do begin
            PixelColor := ColorToRGB(BM.Canvas.Pixels[l, t]);
            c := Round((((PixelColor shr 16) + ((PixelColor shr 8) and $00FF) +
                         (PixelColor and $0000FF)) div 3)) div 2 + 96;
            BM.Canvas.Pixels[l, t] := RGB(c, c, c);
          end;
        Canvas.Draw(x, y, BM);
      finally
        BM.Free;
      end;
    end;
  end
  else if FButtonCaption <> '' then begin
    Canvas.Font := Self.Font;
    Canvas.Brush.Style := bsClear;
    DrawText(Canvas.Handle, PChar(FButtonCaption), Length(FButtonCaption), r,
             DT_SINGLELINE or DT_VCENTER or DT_CENTER);
  end
  else begin
    Canvas.Brush.Color := clBlack;
    Canvas.Pen.Color := clBlack;
    x := (R.Right - R.Left) div 2 - 6 + R.Left;
    y := (R.Bottom - R.Top) div 2 - 1 + R.Top;
    Canvas.Polygon([Point(x + 4, y), Point(x + 8, y), Point(x + 6, y + 2)]);
  end;
  ExcludeClipRect(Canvas.Handle, ClientWidth - FBtnWidth, 0, ClientWidth, ClientHeight);
end;

procedure TzPanelButton.SetButtonWidth(Value: Integer);
begin
  if FBtnWidth <> Value then begin
    FBtnWidth := Value;
    Invalidate;
  end;
end;

function TzPanelButton.GetButtonRect: TRect;
begin
  GetWindowRect(Handle, Result);
  OffsetRect(Result, -Result.Left, -Result.Top);
  Inc(Result.Left, ClientWidth - FBtnWidth);
  OffsetRect(Result, -1, 0);
end;

procedure TzPanelButton.CMMouseEnter(var Message: TMessage);
begin
  FmouseInControl := True;
  PaintButton;
end;

procedure TzPanelButton.CMMouseLeave(var Message: TMessage);
begin
  FmouseInControl := False;
  PaintButton;
end;

procedure TzPanelButton.SetButtonCaption(Value: string);
begin
  if FButtonCaption <> Value then begin
    FButtonCaption := Value;
    PaintButton;
  end;
end;

procedure TzPanelButton.Paint;

  procedure Frame3DBorder (Canvas: TCanvas; Rect: TRect; TopColor, BottomColor: TColor; Width: Integer);

    procedure DoRect;
    var TopRight, BottomLeft: TPoint;
    begin
      with Canvas, Rect do
      begin
        TopRight.X := Right;
        TopRight.Y := Top;
        BottomLeft.X := Left;
        BottomLeft.Y := Bottom;
        Pen.Color := TopColor;
        PolyLine([BottomLeft, TopLeft, TopRight]);
        Pen.Color := BottomColor;
        Dec(BottomLeft.X);
        PolyLine([TopRight, BottomRight, BottomLeft]);
      end;
    end;

  begin
    Canvas.Pen.Width := 1;
    Dec(Rect.Bottom); Dec(Rect.Right);
    while Width > 0 do
    begin
      Dec(Width);
      DoRect;
      InflateRect(Rect, -1, -1);
    end;
    Inc(Rect.Bottom); Inc(Rect.Right);
  end;

var
  memoryBitmap: TBitmap;
  textBounds: TRect;
  Format: UINT;
begin
  textBounds := ClientRect;
  Format := DT_SINGLELINE or DT_VCENTER;
  TextBounds.Right := TextBounds.Right - FBtnWidth;
  case Alignment of
    taLeftJustify: begin
      Format := Format or DT_LEFT;
      TextBounds.Left := TextBounds.Left + 3;
    end;
    taCenter:
      Format := Format or DT_CENTER;
    taRightJustify: begin
      Format := Format or DT_RIGHT;
      TextBounds.Right := TextBounds.Right - 3;
    end;
  end;

  memoryBitmap := TBitmap.Create; // create memory-bitmap to draw flicker-free
  try
    memoryBitmap.Height := ClientRect.Bottom;
    memoryBitmap.Width := ClientRect.Right - FBtnWidth;

    if Enabled then
      memoryBitmap.Canvas.Brush.Color := Self.Color
    else memoryBitmap.Canvas.Brush.Color := clBtnFace;
    memoryBitmap.Canvas.FillRect(ClientRect);

    // Draw Border
    Frame3DBorder(memoryBitmap.Canvas, ClientRect, clBlack, clBlack, 1);

    // Draw Text
    memoryBitmap.Canvas.Font := Self.Font;
    if not Enabled then
      memoryBitmap.Canvas.Font.Color := clBtnShadow;
    memoryBitmap.Canvas.Brush.Style := bsClear;

    if not Enabled then     // Draw Border
      Frame3DBorder(Canvas, ClientRect, clBlack, clBlack, 1);

    DrawText(memoryBitmap.Canvas.Handle, PChar(Caption), Length(Caption), textBounds, Format);
    canvas.CopyRect(ClientRect, memoryBitmap.canvas, ClientRect);
  finally
    memoryBitmap.free; // delete the bitmap
    PaintButton;
  end;
end;

procedure TzPanelButton.SetText(Value: string);
begin
  if Value <> FText then begin
    FText := Value;
    Caption := FText;
    if Assigned(FOnChange) then
      OnChange(Self);
  end;
end;

procedure TzPanelButton.KeyDown(var Key: Word; Shift: TShiftState);
var MyForm : TCustomForm;
begin
   MYForm := GetParentForm( Self );
   case Key of
      VK_DOWN: SendMessage(MYForm.Handle, WM_NEXTDLGCTL, 0, 0);
      VK_UP: SendMessage(MYForm.Handle, WM_NEXTDLGCTL, 1, 0);
      VK_SPACE: if Assigned(FOnBtnClick) then FOnBtnClick(Self)
      else inherited KeyDown(Key, Shift);
   end;
end;

procedure TzPanelButton.SetGlyph(Value: TBitmap);
begin
  if FGlyph = nil then begin
    FGlyph := TBitmap.Create;
    FGlyph.Transparent := True;
  end;
  FGlyph.Assign(Value);
  Invalidate;
end;

function TzPanelButton.GetBitmap: TBitmap;
begin
  if FGlyph = nil then begin
    FGlyph := TBitmap.Create;
    FGlyph.Transparent := True;
  end;
  Result := FGlyph;
end;

procedure TzPanelButton.MouseUp(Button: TMouseButton; Shift: TShiftState; X,Y: Integer);
var r: TRect;
begin
  //inherited MouseUp(Button, Shift, X, Y);
  r := GetButtonRect;
  if Assigned(FOnBtnClick) and (X >= r.Left) then
    FOnBtnClick(Self)
  else if Assigned(FOnMouseUp) then
     FOnMouseUp(Self,Button,Shift,X,Y);
end;

procedure TzPanelButton.Loaded;
begin
  inherited;

  if Enabled then FColor := Color
  //else
  //  Color := clBtnFace;
end;

// Michal Mutl
procedure TzPanelButton.SetButtonColor(const Value: TColor);
begin
  if FBtnColor <> Value then begin
    FBtnColor := Value;
    Invalidate;
  end;
end;

procedure TzPanelButton.SetButtonHLColor(const Value: TColor);
begin
  if FBtnHLColor <> Value then begin
    FBtnHLColor := Value;
    Invalidate;
  end;
end;

procedure Register;
begin
  RegisterComponents('zPage', [TzPanelButton]);
end;

end.
